package SVG::TT::Graph::Bar;

use strict;
use Carp;
use vars qw($VERSION);
$VERSION = '0.03';

use SVG::TT::Graph;
use base qw(SVG::TT::Graph);

# Nasty global! to store the template in, someone patch
# please, so that reading from __DATA__ works even if you
# do it twice from the same script!
my $template;

=head1 NAME

SVG::TT::Graph::Bar - Create presentation quality SVG bar graphs easily

=head1 SYNOPSIS

  use SVG::TT::Graph::Bar;

  my @fields = qw(Jan Feb Mar);
  my @data_sales_02 = qw(12 45 21);

  my $graph = SVG::TT::Graph::Bar->new({
  	'height' => '500',
	'width' => '300',
	'fields' => \@fields,
  });
  
  $graph->add_data({
  	'data' => \@data_sales_02,
	'title' => 'Sales 2002',
  });
  
  print "Content-type: image/svg+xml\r\n";
  print $graph->burn();

=head1 DESCRIPTION

This object aims to allow you to easily create high quality
SVG bar graphs. You can either use the default style sheet
or supply your own. Either way there are many options which can
be configured to give you control over how the graph is
generated - with or without a key, data elements at each point,
title, subtitle etc.

=head1 METHODS

=head2 new()

  use SVG::TT::Graph::Bar;
  
  # Field names along the X axis
  my @fields = qw(Jan Feb Mar);
  
  my $graph = SVG::TT::Graph::Bar->new({
    # Required
    'fields' => \@fields,
  
    # Optional - defaults shown
    'height'            => '500',
    'width'             => '300',
    'show_data_values'  => 1,

    'scale_divisions'   => '',
    'min_scale_value'           => '0',
	'stagger_x_labels'  => 0,
	'bar_gap'           => 1,

    'show_x_labels'     => 1,
    'show_y_labels'     => 1,

    'show_x_title'      => 0,
    'x_title'           => 'X Field names',

    'show_y_title'      => 0,
    'y_title'           => 'Y Scale',

    'show_graph_title'		=> 0,
    'graph_title'           => 'Graph Title',
    'show_graph_subtitle'   => 0,
    'graph_subtitle'        => 'Graph Sub Title',

    # Optional - defaults to using embeded stylesheet
    'style_sheet'       => '/includes/graph.css',
  });

The constructor takes a hash reference, fields (the names for each
field on the X axis) MUST be set, all other values are defaulted to those
shown above - with the exception of style_sheet which defaults
to using the internal style sheet.

=head2 add_data()

  my @data_sales_02 = qw(12 45 21);

  $graph->add_data({
    'data' => \@data_sales_02,
    'title' => 'Sales 2002',
  });

This method allows you to add data to the graph object.
It can be called several times to add more data sets in,
but the likely hood is you should be using SVG::TT::Graph::Line
as it won't look great!

=head2 clear_data()

  my $graph->clear_data();

This method removes all data from the object so that you can
reuse it to create a new graph but with the same config options.

=head2 burn()

  print $graph->burn();

This method processes the template with the data and
config which has been set and returns the resulting SVG.

This method will croak unless at least one data set has
been added to the graph object.

=head2 config methods

  my $value = $graph->method();
  my $confirmed_new_value = $graph->method($value);
  
The following is a list of the methods which are available
to change the config of the graph object after it has been
created.

=over 4

=item height()

Set the height of the graph box, this is the total height
of the SVG box created - not the graph it self which auto
scales to fix the space.

=item width()

Set the width of the graph box, this is the total height
of the SVG box created - not the graph it self which auto
scales to fix the space.

=item style_sheet()

Set the path to an external stylesheet, set to '' if
you want to revert back to using the defaut embeded version.

To create an external stylesheet create a graph using the
default embeded version and copy the stylesheet section to
an external file and edit from there.

=item show_data_values()

Show the value of each element of data on the graph

=item bar_gap()

Whether to have a gap between the bars or not, default
is '1', set to '0' if you don't want gaps.

=item min_scale_value()

The point at which the Y axis starts, defaults to '0',
if set to '' it will default to the minimum data value.

=item scale_divisions()

This defines the gap between markers on the Y axis,
default is a 10th of the max_value, e.g. you will have
10 markers on the Y axis. NOTE: do not set this too
low - you are limited to 999 markers, after that the
graph won't generate.

=item show_x_labels()

Whether to show labels on the X axis or not, defaults
to 1, set to '0' if you want to turn them off.

=item stagger_x_labels()

This puts the labels at alternative levels so if they
are long field names they will not overlap so easily.
Default it '0', to turn on set to '1'.

=item show_y_labels()

Whether to show labels on the Y axis or not, defaults
to 1, set to '0' if you want to turn them off.

=item show_x_title()

Whether to show the title under the X axis labels,
default is 0, set to '1' to show.

=item x_title()

What the title under X axis should be, e.g. 'Months'.

=item show_y_title()

Whether to show the title under the Y axis labels,
default is 0, set to '1' to show.

=item y_title()

What the title under Y axis should be, e.g. 'Sales in thousands'.

=item show_graph_title()

Whether to show a title on the graph, defaults
to 0, set to '1' to show.

=item graph_title()

What the title on the graph should be.

=item show_graph_subtitle()

Whether to show a subtitle on the graph, defaults
to 0, set to '1' to show.

=item graph_subtitle()

What the subtitle on the graph should be.

=item key()

Whether to show a key, defaults to 0, set to
'1' if you want to show it.

=item key_position()

Where the key should be positioned, defaults to
'right', set to 'bottom' if you want to move it.

=back

=head1 NOTES

The default stylesheet handles upto 10 data sets, if you
use more you must create your own stylesheet and add the
additional settings for the extra data sets. You will know
if you go over 10 data sets as they will have no style and
be in black.

=head1 EXAMPLES

For examples look at the project home page
http://leo.cuckoo.org/projects/SVG-TT-Graph/

=head1 EXPORT

None by default.

=head1 ACKNOWLEDGEMENTS

Stephen Morgan for creating the TT template and SVG.

=head1 AUTHOR

Leo Lapworth (LLAP@cuckoo.org)

=head1 SEE ALSO

L<SVG::TT::Graph>,
L<SVG::TT::Graph::Line>,
L<SVG::TT::Graph::BarHorizontal>,
L<SVG::TT::Graph::Pie>,

=cut

sub get_template {
	my $self = shift;
	# read in template
	return $template if $template;
	while(<DATA>) {
		$template .= $_ . "\r\n";
	}
	return $template;
}

sub _init {
	my $self = shift;
	croak "fields was not supplied or is empty" 
	unless defined $self->{'config'}->{fields} 
	&& ref($self->{'config'}->{fields}) eq 'ARRAY'
	&& scalar(@{$self->{'config'}->{fields}}) > 0;
}

sub _set_defaults {
	my $self = shift;
	
	my %default = (
		'width'				=> '500',
		'height'			=> '300',
		'style_sheet'       => '',
	    'show_data_values'  => 1,
	
	    'min_scale_value'   => '0',
		'scale_divisions'	=> '',
		'bar_gap'			=> 1,

	    'show_x_labels'     => 1,
		'stagger_x_labels'	=> 0,
	    'show_y_labels'     => 1,

	    'show_x_title'      => 0,
	    'x_title'           => 'X Field names',
	
	    'show_y_title'      => 0,
	    'y_title'           => 'Y Scale',
	
	    'show_graph_title'		=> 0,
	    'graph_title'			=> 'Graph Title',
	    'show_graph_subtitle'	=> 0,
	    'graph_subtitle'		=> 'Graph Sub Title',
		'key'					=> 0, 
		'key_position'			=> 'right', # bottom or right
	);
	
	while( my ($key,$value) = each %default ) {
		$self->{config}->{$key} = $value;
	}
}

1;
__DATA__
<?xml version="1.0"?>
<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.0//EN"
	"http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd">
[% stylesheet = 'included' %]

[% IF config.style_sheet && config.style_sheet != '' %]
	<?xml-stylesheet href="[% config.style_sheet %]" type="text/css"?>	
[% ELSE %]
	[% stylesheet = 'excluded' %]
[% END %]

<svg width="[% config.width %]" height="[% config.height %]" viewBox="0 0 [% config.width %] [% config.height %]" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">
<!-- \\\\\\\\\\\\\\\\\\\\\\\\\\\\  -->
<!-- Created with SVG::TT::Graph   -->
<!-- Stephen Morgan / Leo Lapworth -->
<!-- ////////////////////////////  -->

[% IF stylesheet == 'excluded' %]
<!-- include default stylesheet if none specified -->
<defs>
<style type="text/css">
<![CDATA[
.svgBackground{
	fill:#ffffff;
}
.graphBackground{
	fill:#f0f0f0;
}

/* graphs titles */
.mainTitle{
	text-anchor: middle;
	fill: #000000;
	font-size: 14px;
	font-family: "Arial", sans-serif;
	font-weight: normal;
}
.subTitle{
	text-anchor: middle;
	fill: #999999;
	font-size: 12px;
	font-family: "Arial", sans-serif;
	font-weight: normal;
}

.axis{
	stroke: #000000;
	stroke-width: 1px;
}

.guideLines{
	stroke: #666666;
	stroke-width: 1px;
	stroke-dasharray: 5 5;
}

.xAxisLabels{
	text-anchor: middle;
	fill: #000000;
	font-size: 12px;
	font-family: "Arial", sans-serif;
	font-weight: normal;
}

.yAxisLabels{
	text-anchor: end;
	fill: #000000;
	font-size: 12px;
	font-family: "Arial", sans-serif;
	font-weight: normal;
}

.xAxisTitle{
	text-anchor: middle;
	fill: #ff0000;
	font-size: 14px;
	font-family: "Arial", sans-serif;
	font-weight: normal;
}

.yAxisTitle{
	fill: #ff0000;
	writing-mode: tb; 
	glyph-orientation-vertical: 0;
	text-anchor: middle;
	font-size: 14px;
	font-family: "Arial", sans-serif;
	font-weight: normal;
}

.dataPointLabel{
	fill: #000000;
	text-anchor:middle;
	font-size: 10px;
	font-family: "Arial", sans-serif;
	font-weight: normal;
}

.staggerGuideLine{
	fill: none;
	stroke: #000000;
	stroke-width: 0.5px;	
}
/* default fill styles for multiple datasets (probably only use a single dataset on this graph though) */
.key1,.fill1{
	fill: #ff0000;
	fill-opacity: 0.2;
	stroke: none;
	stroke-width: 0.5px;	
}
.key2,.fill2{
	fill: #0000ff;
	fill-opacity: 0.2;
	stroke: none;
	stroke-width: 1px;	
}
.key3,.fill3{
	fill: #00ff00;
	fill-opacity: 0.2;
	stroke: none;
	stroke-width: 1px;	
}
.key4,.fill4{
	fill: #ffcc00;
	fill-opacity: 0.2;
	stroke: none;
	stroke-width: 1px;	
}
.key5,.fill5{
	fill: #00ccff;
	fill-opacity: 0.2;
	stroke: none;
	stroke-width: 1px;	
}
.key6,.fill6{
	fill: #ff00ff;
	fill-opacity: 0.2;
	stroke: none;
	stroke-width: 1px;	
}
.key7,.fill7{
	fill: #00ffff;
	fill-opacity: 0.2;
	stroke: none;
	stroke-width: 1px;	
}
.key8,.fill8{
	fill: #ffff00;
	fill-opacity: 0.2;
	stroke: none;
	stroke-width: 1px;	
}
.key9,.fill9{
	fill: #cc6666;
	fill-opacity: 0.2;
	stroke: none;
	stroke-width: 1px;	
}
.key10,.fill10{
	fill: #663399;
	fill-opacity: 0.2;
	stroke: none;
	stroke-width: 1px;	
}
.keyText{
	fill: #000000;
	text-anchor:start;
	font-size: 10px;
	font-family: "Arial", sans-serif;
	font-weight: normal;
}
]]>
</style>
</defs>
[% END %]
<!-- svg bg -->
<rect x="0" y="0" width="[% config.width %]" height="[% config.height %]" class="svgBackground"/>
	
<!-- ///////////////// CALCULATE GRAPH AREA AND BOUNDARIES //////////////// -->
<!-- get dimensions of actual graph area (NOT SVG area) -->
	[% w = config.width %]
	[% h = config.height %]

	<!-- set start/default coords of graph --> 
	[% x = 0 %]
	[% y = 0 %]
	
	[% char_width = 9 %]

<!-- calc min and max values -->
	[% min_value = 99999999999 %]
	[% max_value = 0 %]
	[% max_key_size = 0 %]
	[% FOREACH field = config.fields %]
		[% FOREACH dataset = data %]
			[% IF min_value > dataset.data.$field && dataset.data.$field != '' %]
				[% min_value = dataset.data.$field %]
			[% END %]
			[% IF max_value < dataset.data.$field && dataset.data.$field != '' %]
				[% max_value = dataset.data.$field %]
			[% END %]
		<!-- find largest dataset title for Key size -->
			[% IF max_key_size < dataset.title.length %]
				[% max_key_size = dataset.title.length %]
			[% END %]
		[% END %]
	[% END %]


	
	

<!-- CALC HEIGHT AND Y COORD DIMENSIONS -->
	<!-- reduce height of graph area if there is labelling on x axis -->
	[% IF config.show_x_labels %][% h = h - 20 %][% END %]
	
	<!-- stagger x labels if overlapping occurs -->
	[% stagger = 0 %]
	[% IF config.stagger_x_labels %]
		[% stagger = 17 %]
		[% h = h - stagger %]
	[% END %]
	
	[% IF config.show_x_title %][% h = h - 25 - stagger %][% END %]
	
	<!-- pad top of graph if y axis has data labels so labels do not get chopped off -->
	[% IF config.show_y_labels %][% h = h - 10 %][% y = y + 10 %][% END %]
	
	<!-- reduce height if graph has title or subtitle -->
	[% IF config.show_graph_title %][% h = h - 25 %][% y = y + 25 %][% END %]
	[% IF config.show_graph_subtitle %][% h = h - 10 %][% y = y + 10 %][% END %]
	

	
<!-- reduce graph dimensions if there is a KEY -->
	[% key_box_size = 12 %]
	
	[% IF config.key && config.key_position == 'right' %][% w = w - (max_key_size * (char_width - 1)) - (key_box_size * 3 ) %]
	[% ELSIF config.key && config.key_position == 'bottom' %][% h = h - 50 %]
	[% END %]

	<!-- find start value for scale on y axis -->
	[% IF config.min_scale_value || config.min_scale_value == '0' %]
		[% min_scale_value = config.min_scale_value %]
	[% ELSE %]
		<!-- setting lowest value to be min_value as no min_scale_value defined -->
		[% min_scale_value = min_value %]
	[% END %]
	
	<!-- base line -->
	[% base_line = h + y %]
	
	<!-- how much padding between largest bar and top of graph -->
	[% IF (max_value - min_scale_value) == 0 %]
		[% top_pad = 10 %]
	[% ELSE %]
		[% top_pad = (max_value - min_scale_value) / 20 %]	
	[% END %]	
	
	[% scale_range = (max_value + top_pad) - min_scale_value %]

	<!-- default to 10 scale_divisions if none have been set -->
	[% IF config.scale_divisions %]
		[% scale_division = config.scale_divisions %]
	[% ELSE %]
		[% scale_division = scale_range / 10 FILTER format('%2.0f') %]
	[% END %]

<!-- JUMP THE GUN AND CALC BAR WIDTHS AS THESE ARE USED FOR PADDING IF LARGE X LABELS ARE USED -->
<!-- get number of data points on x scale -->
[% dx = config.fields.size %]

<!-- get distribution width on x axis -->
[% data_widths_x = w / dx %]
[% dw = data_widths_x FILTER format('%2.02f') %]



<!-- CALC WIDTH AND X COORD DIMENSIONS -->
	<!-- reduce width of graph area if there is large labelling on x axis -->
	[% half_label_width = (config.fields.0.length / 2) * char_width %]
	
	[% IF half_label_width > (dw / 2) %]
		<!-- calc difference and pad accordingly -->
		[% space_b4_y_axis = half_label_width - (dw / 2) %]
		<!-- only need to pad the left side if a key is present -->
		[% IF config.key && config.key_position == 'right' %]
			[% w = w - space_b4_y_axis %]
		[% ELSE %]
			<!-- pad both sides -->
			[% w = w - (space_b4_y_axis * 2) %]
		[% END %]
		[% x = x + space_b4_y_axis %]
	[% END %]
	
	<!-- find the string length of max value -->
	[% max_value_length = max_value.length %]
	
	<!-- label width in pixels -->
	[% max_value_length_px = max_value_length * char_width %]
	<!-- If the y labels are shown but the size of the x labels are small, pad for y labels -->
	
	[% IF config.show_y_labels && space_b4_y_axis < max_value_length_px %]
		<!-- allow slightly more padding if small labels -->
		[% IF max_value_length < 2 %]
			[% w = w - (max_value_length * (char_width * 2)) %]
			[% x = x + (max_value_length * (char_width * 2)) %]
		[% ELSE %]
			[% w = w - max_value_length_px %]
			[% x = x + max_value_length_px %]
		[% END %]
	[% END %]
	
	[% IF config.show_y_title && space_b4_y_axis < max_value_length_px %]
		[% w = w - 25 %]
		[% x = x + 25 %]
	[% END %]
		




<!-- //////////////////////////////  BUILD GRAPH AREA ////////////////////////////// -->
<!-- graph bg -->
	<rect x="[% x %]" y="[% y %]" width="[% w %]" height="[% h %]" class="graphBackground"/>

<!-- axis -->
	<path d="M[% x %] [% y %] v[% h %]" class="axis" id="xAxis"/>
	<path d="M[% x %] [% base_line %] h[% w %]" class="axis" id="yAxis"/>

<!-- //////////////////////////////  AXIS DISTRIBUTIONS //////////////////////////// -->
	<!-- get number of data points on x scale -->
	[% dx = config.fields.size %]
		<!-- ensure x_data_points butt up to edge of graph -->
		[%# dx = dx - 1 %]
		[% dx = dx %]
<!-- get distribution width on x axis -->
[% data_widths_x = w / dx %]
[% dw = data_widths_x.match('(\d+[\.\d\d])').0 %]

[% i = dw %]
[% count = 0 %]

[% IF config.bar_gap %]
	[% bar_gap = 10 %]
[% ELSE %]
	[% bar_gap = 0 %]
[% END %]

[% stagger_count = 0 %]
<!-- x axis labels -->
[% IF config.show_x_labels %]
	[% FOREACH field = config.fields %]
		[% IF count == 0 %]
<text x="[% x + (dw / 2) - (bar_gap / 2) %]" y="[% base_line + 15 %]" class="xAxisLabels">[% field %]</text>
		[% i = i - dw %]
		[% ELSE %]
			[% IF stagger_count == 2 %]
				<text x="[% x + i + (dw / 2) - (bar_gap / 2) %]" y="[% base_line + 15 %]" class="xAxisLabels">[% field %]</text>
				[% stagger_count = 0 %]
			[% ELSE %]
				<text x="[% x + i + (dw / 2) - (bar_gap / 2) %]" y="[% base_line + 15 + stagger %]" class="xAxisLabels">[% field %]</text>
				<path d="M[% x + i + (dw / 2) - (bar_gap / 2) %] [% base_line %], v[% stagger %]" class="staggerGuideLine" />
			[% END %]
		[% END %]
	[% i = i + dw %]
	[% count = count + 1 %]
	[% stagger_count = stagger_count + 1 %]
	[% END %]
[% END %]


<!-- distribute Y scale -->
[% dy = scale_range / scale_division %]
<!-- ensure y_data_points butt up to edge of graph -->
[% y_marker_height = h / dy %]
[% dy = y_marker_height.match('(\d+[\.\d\d])').0 %]
[% count = 0 %]
[% y_value = min_scale_value %]
[% IF config.show_y_labels %]
	[% WHILE (dy * count) < h %]
		[% IF count == 0 %]
		<!-- no stroke for first line -->
			<text x="[% x - 5 %]" y="[% base_line - (dy * count) %]" class="yAxisLabels">[% y_value %]</text>
		[% ELSE %]
			<text x="[% x - 5 %]" y="[% base_line - (dy * count) %]" class="yAxisLabels">[% y_value %]</text>
			<path d="M[% x %] [% base_line - (dy * count) %] h[% w %]" class="guideLines"/>
		[% END %]
		[% y_value = y_value + scale_division %]
		[% count = count + 1 %]
	[% END %]
[% END %]




<!-- //////////////////////////////  AXIS TITLES ////////////////////////////// -->

<!-- x axis title -->
	[% IF config.show_x_title %]
		[% IF !config.show_x_labels %]
			[% y_xtitle = 15 %]
		[% ELSE %]
			[% y_xtitle = 35 %]
		[% END %]
		<text x="[% (w / 2) + x %]" y="[% h + y + y_xtitle + stagger %]" class="xAxisTitle">[% config.x_title %]</text>
	[% END %]	

<!-- y axis title -->
	[% IF config.show_y_title %]
			<text x="10" y="[% (h / 2) + y %]" class="yAxisTitle">[% config.y_title %]</text>
	[% END %]




<!-- //////////////////////////////  SHOW DATA ////////////////////////////// -->
[% bar_width = dw - bar_gap %]

[% divider = dy / scale_division %]
<!-- data points on graph -->
[% IF config.show_data_points || config.show_data_values %]
	[% xcount = 0 %]
	
	[% FOREACH field = config.fields %]
		[% dcount = 1 %]
		<!-- find the lowest data value for each dataset -->

		[% FOREACH dataset = data %]
			<path d="M[% (dw * xcount) + x %] [% base_line %] V[% base_line - (dataset.data.$field * divider) %] h[% bar_width %] V[% base_line %] Z" class="fill[% dcount %]"/>
			[% IF config.show_data_values %]
				<text x="[% (dw * xcount) + x + (dw / 2) - (bar_gap / 2) %]" y="[% base_line - (dataset.data.$field * divider) - 6 %]" class="dataPointLabel">[% dataset.data.$field %]</text>
			[% END %]
			[% dcount = dcount + 1 %]	
		[% END %]
		[% xcount = xcount + 1 %]		
	[% END %]
[% END %]

<!-- //////////////////////////////// KEY /////// ////////////////////////// -->
[% key_count = 1 %]
[% key_padding = 5 %]
[% IF config.key && config.key_position == 'right' %]
	[% FOREACH dataset = data %]
		<rect x="[% x + w + 20 %]" y="[% y + (key_box_size * key_count) + (key_count * key_padding) %]" width="[% key_box_size %]" height="[% key_box_size %]" class="key[% key_count %]"/>
		<text x="[% x + w + 20 + key_box_size + key_padding %]" y="[% y + (key_box_size * key_count) + (key_count * key_padding) + key_box_size %]" class="keyText">[% dataset.title %]</text>
		[% key_count = key_count + 1 %]
	[% END %]
[% ELSIF config.key && config.key_position == 'bottom' %]
	<!-- calc y position of start of key -->
	[% y_key = base_line %]
	[% IF config.show_x_labels %][% y_key = base_line + 20 %][% END %]
	[% IF config.show_x_title %][% y_key = base_line + 25 %][% END %]
	[% y_key_start = y_key %]
	[% x_key = x %]
	[% FOREACH dataset = data %]
		[% IF key_count == 4 || key_count == 7 || key_count == 10 %]
		<!-- wrap key every 3 entries -->
			[% x_key = x_key + 200 %]
			[% y_key = y_key - (key_box_size * 4) - 2 %]
		[% END %]
		<rect x="[% x_key %]" y="[% y_key + (key_box_size * key_count) + (key_count * key_padding) %]" width="[% key_box_size %]" height="[% key_box_size %]" class="key[% key_count %]"/>

		<text x="[% x_key + key_box_size + key_padding %]" y="[% y_key + (key_box_size * key_count) + (key_count * key_padding) + key_box_size %]" class="keyText">[% dataset.title %]</text>
		[% key_count = key_count + 1 %]
	[% END %]
	
[% END %]
	
<!-- //////////////////////////////// MAIN TITLES ////////////////////////// -->

<!-- main graph title -->
	[% IF config.show_graph_title %]
		<text x="[% config.width / 2 %]" y="15" class="mainTitle">[% config.graph_title %]</text>
	[% END %]

<!-- graph sub title -->
	[% IF config.show_graph_subtitle %]
		[% IF config.show_graph_title %]
			[% y_subtitle = 30 %]
		[% ELSE %]
			[% y_subtitle = 15 %]
		[% END %]
		<text x="[% config.width / 2 %]" y="[% y_subtitle %]" class="subTitle">[% config.graph_subtitle %]</text>
	[% END %]
</svg>